// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#pragma once

#include <cstdint>

/**
 * A type for absolute offsets in a file.
 */
typedef uint64_t offset_type;
