// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_XIPH_TAGS_HXX
#define MPD_XIPH_TAGS_HXX

#include "tag/Table.hxx"

extern const struct tag_table xiph_tags[];

#endif
