// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_DECODER_FLAC_H
#define MPD_DECODER_FLAC_H

extern const struct DecoderPlugin flac_decoder_plugin;
extern const struct DecoderPlugin oggflac_decoder_plugin;

#endif
