// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_STORAGE_NFS_HXX
#define MPD_STORAGE_NFS_HXX

struct StoragePlugin;

extern const StoragePlugin nfs_storage_plugin;

#endif
