// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef NET_FEATURES_HXX
#define NET_FEATURES_HXX

/* feature macros are defined in config.h */
#include "config.h"

#endif
