// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_FFMPEG_INPUT_PLUGIN_HXX
#define MPD_FFMPEG_INPUT_PLUGIN_HXX

/**
 * An input plugin based on libavformat's "avio" library.
 */
extern const struct InputPlugin input_plugin_ffmpeg;

#endif
