// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_TAG_ID3MUSICBRAINZ_HXX
#define MPD_TAG_ID3MUSICBRAINZ_HXX

extern const struct tag_table musicbrainz_txxx_tags[];

#endif
