// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_PULSE_DOMAIN_HXX
#define MPD_PULSE_DOMAIN_HXX

class Domain;

extern const Domain pulse_domain;

#endif
