// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef INPUT_QOBUZ_HXX
#define INPUT_QOBUZ_HXX

extern const struct InputPlugin qobuz_input_plugin;

#endif
