// SPDX-License-Identifier: GPL-2.0-or-later
// Copyright The Music Player Daemon Project

#ifndef MPD_ROUTE_FILTER_PLUGIN_HXX
#define MPD_ROUTE_FILTER_PLUGIN_HXX

struct FilterPlugin;

extern const FilterPlugin route_filter_plugin;

#endif
